@@ -13,11 +13,16 @@ from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.files.base import ContentFile
 from django.test import TestCase, override_settings
-from django.utils.six.moves.urllib import parse as urlparse
 from django.utils.timezone import is_aware, utc
 
 from storages.backends import s3boto3
 
+try:
+    from django.utils.six.moves.urllib import parse as urlparse
+except ImportError:
+    from urllib import parse as urlparse
+
+
 try:
     from unittest import mock
 except ImportError:  # Python 3.2 and below
@@ -116,7 +121,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
 
         obj = self.storage.bucket.Object.return_value
         obj.upload_fileobj.assert_called_with(
-            content.file,
+            content,
             ExtraArgs={
                 'ContentType': 'text/plain',
                 'ACL': self.storage.default_acl,
@@ -135,7 +140,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
 
         obj = self.storage.bucket.Object.return_value
         obj.upload_fileobj.assert_called_with(
-            content.file,
+            content,
             ExtraArgs={
                 'ContentType': 'text/plain',
                 'ACL': 'private',
@@ -154,7 +159,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
 
         obj = self.storage.bucket.Object.return_value
         obj.upload_fileobj.assert_called_with(
-            content.file,
+            content,
             ExtraArgs={
                 'ContentType': 'image/jpeg',
                 'ACL': self.storage.default_acl,
@@ -170,7 +175,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.save(name, content)
         obj = self.storage.bucket.Object.return_value
         obj.upload_fileobj.assert_called_with(
-            content.file,
+            content,
             ExtraArgs={
                 'ContentType': 'application/octet-stream',
                 'ContentEncoding': 'gzip',
@@ -273,6 +278,65 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         multipart.complete.assert_called_once_with(
             MultipartUpload={'Parts': [{'ETag': '123', 'PartNumber': 1}]})
 
+    def test_storage_open_no_write(self):
+        """
+        Test opening file in write mode and closing without writing.
+
+        A file should be created as by obj.put(...).
+        """
+        name = 'test_open_no_write.txt'
+
+        # Set the encryption flag used for puts
+        self.storage.encryption = True
+        self.storage.reduced_redundancy = True
+        self.storage.default_acl = 'public-read'
+
+        file = self.storage.open(name, 'w')
+        self.storage.bucket.Object.assert_called_with(name)
+        obj = self.storage.bucket.Object.return_value
+        obj.load.side_effect = ClientError({'Error': {},
+                                            'ResponseMetadata': {'HTTPStatusCode': 404}},
+                                           'head_bucket')
+
+        # Set the name of the mock object
+        obj.key = name
+
+        # Save the internal file before closing
+        file.close()
+
+        obj.load.assert_called_once_with()
+        obj.put.assert_called_once_with(
+            ACL='public-read',
+            Body=b"",
+            ContentType='text/plain',
+            ServerSideEncryption='AES256',
+            StorageClass='REDUCED_REDUNDANCY'
+        )
+
+    def test_storage_open_no_overwrite_existing(self):
+        """
+        Test opening an existing file in write mode and closing without writing.
+        """
+        name = 'test_open_no_overwrite_existing.txt'
+
+        # Set the encryption flag used for puts
+        self.storage.encryption = True
+        self.storage.reduced_redundancy = True
+        self.storage.default_acl = 'public-read'
+
+        file = self.storage.open(name, 'w')
+        self.storage.bucket.Object.assert_called_with(name)
+        obj = self.storage.bucket.Object.return_value
+
+        # Set the name of the mock object
+        obj.key = name
+
+        # Save the internal file before closing
+        file.close()
+
+        obj.load.assert_called_once_with()
+        obj.put.assert_not_called()
+
     def test_storage_write_beyond_buffer_size(self):
         """
         Test writing content that exceeds the buffer size
@@ -324,8 +388,8 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         )
         part = multipart.Part.return_value
         uploaded_content = ''.join(
-            (args_list[1]['Body'].decode('utf-8')
-                for args_list in part.upload.call_args_list)
+            args_list[1]['Body'].decode('utf-8')
+            for args_list in part.upload.call_args_list
         )
         self.assertEqual(uploaded_content, written_content)
         multipart.complete.assert_called_once_with(
@@ -478,7 +542,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.assertIs(
             settings.USE_TZ,
             is_aware(self.storage.get_modified_time(name)),
-            '%s datetime object expected from get_modified_time() when USE_TZ=%s' % (
+            '{} datetime object expected from get_modified_time() when USE_TZ={}'.format(
                 ('Naive', 'Aware')[settings.USE_TZ],
                 settings.USE_TZ
             )
@@ -564,7 +628,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         message = (
             "The acl argument of S3Boto3Storage is deprecated. Use argument "
             "default_acl or setting AWS_DEFAULT_ACL instead. The acl argument "
-            "will be removed in version 2.0."
+            "will be removed in version 1.10."
         )
         assert str(w[-1].message) == message
 
@@ -576,6 +640,41 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         message = (
             "The bucket argument of S3Boto3Storage is deprecated. Use argument "
             "bucket_name or setting AWS_STORAGE_BUCKET_NAME instead. The bucket "
-            "argument will be removed in version 2.0."
+            "argument will be removed in version 1.10."
         )
         assert str(w[-1].message) == message
+
+    def test_deprecated_default_acl(self):
+        with warnings.catch_warnings(record=True) as w:
+            s3boto3.S3Boto3Storage()
+        assert len(w) == 1
+        message = (
+            "The default behavior of S3Boto3Storage is insecure and will change "
+            "in django-storages 1.10. By default files and new buckets are saved "
+            "with an ACL of 'public-read' (globally publicly readable). Version 1.10 will "
+            "default to using the bucket's ACL. To opt into the new behavior set "
+            "AWS_DEFAULT_ACL = None, otherwise to silence this warning explicitly "
+            "set AWS_DEFAULT_ACL."
+        )
+        assert str(w[-1].message) == message
+
+    def test_deprecated_autocreate_bucket(self):
+        with override_settings(AWS_DEFAULT_ACL=None), warnings.catch_warnings(record=True) as w:
+            s3boto3.S3Boto3Storage(auto_create_bucket=True)
+        assert len(w) == 1
+        assert issubclass(w[-1].category, DeprecationWarning)
+        message = (
+            "Automatic bucket creation will be removed in version 1.10. It encourages "
+            "using overly broad credentials with this library. Either create it before "
+            "manually or use one of a myriad of automatic configuration management tools. "
+            "Unset AWS_AUTO_CREATE_BUCKET (it defaults to False) to silence this warning."
+        )
+        assert str(w[-1].message) == message
+
+    def test_deprecated_default_acl_override_class_variable(self):
+        class MyStorage(s3boto3.S3Boto3Storage):
+            default_acl = "private"
+
+        with warnings.catch_warnings(record=True) as w:
+            MyStorage()
+        assert len(w) == 0
